#######################################################################################################
##                             Compte B-spline matrices and derivatives.                             ##
#######################################################################################################

## Arguments
## x = vector of observations
## xl, xr = min(x) and max(x)
## ndx = number of equally spaced knots
## deg = B-spline degree
## max_derivs = maximum order of derivative to compute
## sparse = logical if a sparse representation is required

## Output
## A list with each element refering to derivative of order 0,1,..., max_derivs 

basis_array = function(t, tl = min(t), tr = max(t), ndt = 10, deg = 3, max_derivs = 0, sparse = FALSE)
{
  dt = (tr - tl) / ndt
  knots = seq(tl - deg * dt, tr + deg * dt, by = dt)

  B <- list()
  for (i in 1:(max_derivs + 1))
  {
     B[[i]] = splineDesign(knots = knots,
                           x = t,
                           ord = deg + 1,
                           derivs = rep(i - 1, length(t)),
                           outer.ok = TRUE,
                           sparse = sparse)
  }

  B
}