#######################################################################################################
##    Compute spline coefficients and optimization criterion for ODE parameters in QL-ODE-P-splines  ##
## (Frasso, Jaeger and Lambert, 2015)                                                                ##
#######################################################################################################

## Function arguments:
## theta = current estimates ODE parameters
## tau = current estimate for the precision of measurements
## ck = current estimates spline coefficients
## length_t = length grid B-spline for penalty matrix
## seq_t = grid of time points (B-spline  for penalty)
## B_seq_t = list of B-splines terms for the construction of the penalty
## min_t, max_t = min and max values of observed times
## regul = (optional) extra regularization parameter (default = 0)

#--------------------- function to compute the estimation of the spline coefficients ------------------#

c_hat_function = function(theta, tau, gamma, ck, length_t, seq_t, B_seq_t, regul = 0)
 {
  pen = pen_fun(theta, ck, length_t, seq_t, B_seq_t, min_t = min_t, max_t = max_t)

  R = pen $ R
  r = pen $ r

  c_hat = solve(tau * tB_B + gamma * R + regul * diag(ncol(R)), tol = 1e-25) %*% (tau * tB_y - gamma * r)

  return(list(as.numeric(c_hat), pen))
 }


#----------------- function to compute the H criterion for estimation of ODE parameters ----------------#

H_fun = function(theta, tau, gamma, ck, length_t, seq_t, B_seq_t, regul = 0)
{
 ckp1 = c_hat_function(theta, tau, gamma, ck, length_t, seq_t, B_seq_t, regul)[[1]]

 y_star = as.vector(B_0 %*% ckp1)

 H = - tau * 0.5 * sum( (y_obs - y_star) ^ 2)

 return(H)
}

 