#######################################################################################################
##          Penalty function for QL-ODE-P-splines (Frasso, Jaeger and Lambert, 2015)                 ##
## This function returns the terms to build a quasilinearized penalty for the ODE problem:           ##
##                             dx = (theta_1 - 2 * theta2 * t) * x^2                                 ##  
#######################################################################################################

## Function arguments:
## theta = current estimates ODE parameters
## ck = current estimates spline coefficients
## length_t = length grid B-spline for penalty matrix
## seq_t = grid of time points (B-spline  for penalty)
## B_seq_t = list of B-splines terms for the construction of the penalty
## min_t, max_t = min and max values of observed times

pen_fun = function(theta, ck, length_t, seq_t, B_seq_t, min_t, max_t)
{
 theta1 = theta[1]
 theta2 = theta[2]

 B_x_0_seq_t = as.matrix(B_seq_t[[1]])
 B_x_1_seq_t = as.matrix(B_seq_t[[2]])
 B_x_0_seq_t_times_ck = B_x_0_seq_t %*% ck

 w1 = rep(1, length_t)
 w2 = as.vector(- (theta1 - 2 * theta2 * seq_t) * 2 * B_x_0_seq_t_times_ck)

 H = (w1 * B_x_1_seq_t + w2 * B_x_0_seq_t)
 h = as.vector( (theta1 - 2 * theta2 * seq_t) * (B_x_0_seq_t_times_ck ^ 2) )
 
 del = rep((max_t - min_t) / (length_t - 1), length_t - 2)
 
 W = c(del[1]/2, del, del[1]/2)

 R = t(W * H) %*% H

 r = t(W * H) %*% h

 l = t(W * h) %*% h

 return(list(R = R, r = c(r), l = c(l)))
}